@@ -66,6 +66,7 @@ class ChatBanner(Gtk.Box, EventHelper, SignalManager):
         Gtk.Box.__init__(self)
         EventHelper.__init__(self)
         SignalManager.__init__(self)
+        self.set_name("ChatBanner")
 
         self._client: types.Client | None = None
         self._contact: types.ChatContactT | None = None
@@ -73,11 +74,12 @@ class ChatBanner(Gtk.Box, EventHelper, SignalManager):
         self._last_message_from_phone: set[BareContact] = set()
 
         self._account_badge = AccountBadge(bind_setting=True)
-        self._voice_requests_button = VoiceRequestsButton()
-
-        self._additional_items_box.append(self._voice_requests_button)
+        self._account_badge.set_valign(Gtk.Align.CENTER)
         self._additional_items_box.append(self._account_badge)
 
+        self._voice_requests_button = VoiceRequestsButton()
+        self._additional_items_box.append(self._voice_requests_button)
+
         self._avatar_button.set_create_popup_func(self._on_avatar_clicked)
 
         hide_roster = app.settings.get("hide_groupchat_occupants_list")
@@ -281,8 +283,8 @@ class ChatBanner(Gtk.Box, EventHelper, SignalManager):
     def _update_avatar(self) -> None:
         scale = app.window.get_scale_factor()
         assert self._contact
-        texture = self._contact.get_avatar(AvatarSize.CHAT, scale)
-        self._avatar_image.set_pixel_size(AvatarSize.CHAT)
+        texture = self._contact.get_avatar(AvatarSize.CHAT_BANNER, scale)
+        self._avatar_image.set_pixel_size(AvatarSize.CHAT_BANNER)
         self._avatar_image.set_from_paintable(texture)
 
     def _update_avatar_button_sensitivity(self) -> None:
