@@ -25,9 +25,9 @@ from gajim.common.events import SearchResultReceivedEvent
 from gajim.common.i18n import _
 
 from gajim.gtk.assistant import Assistant
-from gajim.gtk.assistant import ErrorPage
-from gajim.gtk.assistant import Page
-from gajim.gtk.assistant import ProgressPage
+from gajim.gtk.assistant import AssistantErrorPage
+from gajim.gtk.assistant import AssistantPage
+from gajim.gtk.assistant import AssistantProgressPage
 from gajim.gtk.dataform import DataFormWidget
 from gajim.gtk.menus import get_component_search_menu
 from gajim.gtk.util.misc import ensure_not_destroyed
@@ -75,8 +75,6 @@ class ComponentSearch(Assistant):
 
         self._client.get_module("Search").request_search_fields(self._jid)
 
-        self.show_all()
-
     @overload
     def get_page(self, name: Literal["prepare"]) -> RequestForm: ...
 
@@ -89,7 +87,7 @@ class ComponentSearch(Assistant):
     @overload
     def get_page(self, name: Literal["error"]) -> Error: ...
 
-    def get_page(self, name: str) -> Page:
+    def get_page(self, name: str) -> AssistantPage:
         return self._pages[name]
 
     def _on_button_clicked(self, _assistant: Assistant, button_name: str) -> None:
@@ -127,13 +125,13 @@ class ComponentSearch(Assistant):
         self.show_page("result")
 
     def _cleanup(self) -> None:
-        self.unregister_events()
+        Assistant._cleanup(self)
         self._destroyed = True
 
 
-class RequestForm(ProgressPage):
+class RequestForm(AssistantProgressPage):
     def __init__(self):
-        ProgressPage.__init__(self)
+        AssistantProgressPage.__init__(self)
         self.set_title(_("Request Search Form"))
         self.set_text(_("Requesting search form from server"))
 
@@ -141,9 +139,9 @@ class RequestForm(ProgressPage):
         return ["close"]
 
 
-class SearchForm(Page):
+class SearchForm(AssistantPage):
     def __init__(self) -> None:
-        Page.__init__(self)
+        AssistantPage.__init__(self)
         self.title = _("Search")
 
         self.complete = False
@@ -191,9 +189,9 @@ class SearchForm(Page):
         return "search"
 
 
-class Result(Page):
+class Result(AssistantPage):
     def __init__(self) -> None:
-        Page.__init__(self)
+        AssistantPage.__init__(self)
         self.title = _("Search Result")
 
         self._jid_col: int | None = None
@@ -288,11 +286,13 @@ class Result(Page):
         _n_press: int,
         x: float,
         y: float,
-    ) -> int:
+    ) -> None:
         assert self._treeview is not None
         path = self._treeview.get_path_at_pos(int(x), int(y))
         if path is None:
-            return False
+            return
+
+        gesture_click.set_state(Gtk.EventSequenceState.CLAIMED)
 
         path, _column, _x, _y = path
         store = self._treeview.get_model()
@@ -309,7 +309,6 @@ class Result(Page):
         self._popover_menu.set_menu_model(menu)
         self._popover_menu.set_pointing_to_coord(x, y)
         self._popover_menu.popup()
-        return Gdk.EVENT_STOP
 
     def get_visible_buttons(self) -> list[str]:
         return ["close", "new-search"]
@@ -318,9 +317,9 @@ class Result(Page):
         return "close"
 
 
-class Error(ErrorPage):
+class Error(AssistantErrorPage):
     def __init__(self) -> None:
-        ErrorPage.__init__(self)
+        AssistantErrorPage.__init__(self)
         self.set_title(_("Error"))
         self.set_heading(_("An error occurred"))
 
